@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # coding: utf-8
 
-# In[2]:
+# In[124]:
 
 
 try:
@@ -12,10 +12,10 @@ except:
         pass
 
 
-# In[3]:
+# In[125]:
 
 
-version = '0.63'
+version = '0.67'
 
 import re
 import tempfile
@@ -25,15 +25,15 @@ import glob
 from collections import OrderedDict
 import base64
 import shutil
-import time
 import os
 import stat
 import pickle
+import time
 
 
 # # Helper Functions
 
-# In[4]:
+# In[126]:
 
 
 def is_testing():
@@ -43,7 +43,7 @@ def is_testing():
         return False
 
 
-# In[5]:
+# In[127]:
 
 
 def isnotebook():
@@ -54,7 +54,7 @@ def isnotebook():
     return False
 
 
-# In[6]:
+# In[128]:
 
 
 def testPrint(*args):
@@ -62,9 +62,37 @@ def testPrint(*args):
         print(*args)
 
 
+# # Timing
+
+# In[129]:
+
+
+timeSinceLastCallDict = {}
+
+
+# In[130]:
+
+
+def timeSinceLastCall(name):
+    t = time.time()
+    
+    last_t = timeSinceLastCallDict.get(name, 0)
+    timeSinceLastCallDict[name] = t
+    
+    time_s = "{:.2E}".format(t - last_t) if last_t > 0 else "-1"
+    return name + ": " + time_s
+
+
+# In[131]:
+
+
+if is_testing():
+    print(timeSinceLastCall('test marker'))
+
+
 # # Requiremtns
 
-# In[7]:
+# In[132]:
 
 
 requirements = []
@@ -96,7 +124,7 @@ import yaml
 
 # # Strings
 
-# In[8]:
+# In[133]:
 
 
 def sFloat0(f): return "{:0.0f}".format(f)
@@ -126,7 +154,7 @@ if is_testing():
     print(sFloatSci5(1.234567))
 
 
-# In[110]:
+# In[134]:
 
 
 def strUuid4():
@@ -137,14 +165,14 @@ def strUuid4():
     return uuid
 
 
-# In[112]:
+# In[135]:
 
 
 if is_testing():
     print(strUuid4())
 
 
-# In[113]:
+# In[136]:
 
 
 def strRand(n, num=True, upperCase=True, lowerCase=True):
@@ -156,7 +184,7 @@ def strRand(n, num=True, upperCase=True, lowerCase=True):
     return ''.join(random.choice(pool) for _ in range(n))
 
 
-# In[115]:
+# In[137]:
 
 
 if is_testing():
@@ -165,9 +193,42 @@ if is_testing():
     print(strRand(20, upperCase=False, lowerCase=False))
 
 
+# In[138]:
+
+
+def strMd5Pickle4(obj):
+    # Must convet to hashable object; This function uses pickle for that
+    import hashlib
+    md5 = hashlib.md5(pickle.dumps(obj, protocol=4))
+    return md5.hexdigest()
+
+
+# In[139]:
+
+
+if is_testing():
+    print(strMd5Pickle4(['a', 'b']))
+
+
+# In[140]:
+
+
+def strMd5String(s):
+    import hashlib
+    md5 = hashlib.md5(s.encode('utf-8'))
+    return md5.hexdigest()
+
+
+# In[141]:
+
+
+if is_testing():
+    print(strMd5String('s'))
+
+
 # # Process / Thread
 
-# In[9]:
+# In[142]:
 
 
 def mapThread(func, args, n=8):
@@ -190,14 +251,14 @@ if is_testing():
 
 # # Lists
 
-# In[10]:
+# In[143]:
 
 
 def liChunks(l, chunk_size):
     return [l[i:i + chunk_size] for i in range(0, len(l), chunk_size)]
 
 
-# In[11]:
+# In[144]:
 
 
 if is_testing():
@@ -206,7 +267,7 @@ if is_testing():
     print(liChunks(['a', 'b', 'c', 'd', 'e', 'f'], 3))
 
 
-# In[12]:
+# In[145]:
 
 
 def liSplit(a, number_of_splits):
@@ -214,7 +275,7 @@ def liSplit(a, number_of_splits):
     return [a[start:end] for start, end in zip(idxs[:-1], idxs[1:])]
 
 
-# In[13]:
+# In[146]:
 
 
 if is_testing():
@@ -226,7 +287,7 @@ if is_testing():
     print(liSplit(['a', 'b', 'c', 'd', 'e'], 6))
 
 
-# In[14]:
+# In[147]:
 
 
 def liJoin(l):
@@ -236,14 +297,14 @@ def liJoin(l):
     return out
 
 
-# In[15]:
+# In[148]:
 
 
 if is_testing():
     print(liJoin([['a', 'b', 'c'], ['d', 'e', 'f']]))
 
 
-# In[16]:
+# In[149]:
 
 
 def liFlatten(l):
@@ -260,7 +321,7 @@ def liFlatten(l):
         return l
 
 
-# In[17]:
+# In[150]:
 
 
 if is_testing():
@@ -270,7 +331,7 @@ if is_testing():
 
 # # List of Strings
 
-# In[18]:
+# In[151]:
 
 
 regex_cache = {}
@@ -280,35 +341,35 @@ def sListFilter(l, re_include=".*", re_exclude='^\b$'):
     return [e for e in l if regex_cache[re_include].search(e) and not regex_cache[re_exclude].search(e)]
 
 
-# In[19]:
+# In[152]:
 
 
 if is_testing():
     sListFilter(['good x', 'other good x', 'without the letter before y'])
 
 
-# In[20]:
+# In[153]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y'], re_include='x'))
 
 
-# In[21]:
+# In[154]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+'))
 
 
-# In[22]:
+# In[155]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+', re_exclude='z'))
 
 
-# In[23]:
+# In[156]:
 
 
 def lCsv(l, force_oneline=False):
@@ -324,21 +385,21 @@ def lCsv(l, force_oneline=False):
     return ", ".join([str(e) for e in l])
 
 
-# In[24]:
+# In[157]:
 
 
 if is_testing():
     print(lCsv(['a', 'b', 1, 3.14]))
 
 
-# In[25]:
+# In[158]:
 
 
 if is_testing():
     print(lCsv([['a', 'b'], 1, 3.14]))
 
 
-# In[26]:
+# In[159]:
 
 
 if is_testing():
@@ -347,7 +408,7 @@ if is_testing():
 
 # # Text files
 
-# In[27]:
+# In[160]:
 
 
 def txtread(path):
@@ -368,7 +429,7 @@ def txtwrite(path, txt):
             return f.write(txt)
 
 
-# In[28]:
+# In[161]:
 
 
 if is_testing():
@@ -377,21 +438,21 @@ if is_testing():
 
 # # YAML files
 
-# In[29]:
+# In[162]:
 
 
 def yamlwrite(path, obj):
     txtwrite(path, yaml.dump(obj))
 
 
-# In[30]:
+# In[163]:
 
 
 def yamlread(path):
     return yaml.safe_load(txtread(path=path))
 
 
-# In[31]:
+# In[164]:
 
 
 if is_testing():
@@ -404,7 +465,7 @@ if is_testing():
 
 # # List of Strings
 
-# In[32]:
+# In[165]:
 
 
 regex_cache = {}
@@ -416,35 +477,35 @@ def sListFilter(listOfStrings, re_include=".*", re_exclude='^\b$'):
 
 # ### Examples
 
-# In[33]:
+# In[166]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y']))
 
 
-# In[34]:
+# In[167]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y'], re_include='x'))
 
 
-# In[35]:
+# In[168]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+'))
 
 
-# In[36]:
+# In[169]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+', re_exclude='z'))
 
 
-# In[37]:
+# In[170]:
 
 
 if is_testing():
@@ -453,7 +514,7 @@ if is_testing():
 
 # # GCP Google Cloud Platform
 
-# In[38]:
+# In[171]:
 
 
 import warnings
@@ -477,7 +538,7 @@ def _gsBlob(path):
     return blob
 
 
-# In[39]:
+# In[172]:
 
 
 def gsGsToHttp(path):
@@ -487,7 +548,7 @@ def gsHttpToGs(path):
     return path.replace("https://storage.cloud.google.com/", "gs://").split("?")[0] # "?" is for possible "?authuser=0"
 
 
-# In[40]:
+# In[173]:
 
 
 def gsLs(path):
@@ -497,7 +558,7 @@ def gsLs(path):
     return ['gs://' + bucket + '/' + blob.name for blob in client.list_blobs(prefix=prefix)]
 
 
-# In[41]:
+# In[174]:
 
 
 def gsExists(path):
@@ -507,7 +568,7 @@ def gsExists(path):
     return client.blob(prefix).exists()
 
 
-# In[42]:
+# In[175]:
 
 
 def gsUploadFile(gsPath, path):
@@ -515,7 +576,7 @@ def gsUploadFile(gsPath, path):
     blob.upload_from_filename(path)
 
 
-# In[43]:
+# In[176]:
 
 
 def gsTxtread(path):
@@ -527,7 +588,7 @@ def gsTxtread(path):
     return blob.download_as_string().decode()
 
 
-# In[44]:
+# In[177]:
 
 
 def gsTxtwrite(path, s):
@@ -535,7 +596,7 @@ def gsTxtwrite(path, s):
     return blob.upload_from_string(s)
 
 
-# In[45]:
+# In[178]:
 
 
 def gsImwrite(path, img):
@@ -544,7 +605,7 @@ def gsImwrite(path, img):
     blob.upload_from_string(encimg.tobytes())
 
 
-# In[46]:
+# In[179]:
 
 
 def gsImread(path):
@@ -553,7 +614,7 @@ def gsImread(path):
     return cv2.imdecode(raw, 1)[:, :, [2, 1, 0]]
 
 
-# In[47]:
+# In[180]:
 
 
 def gsPklWrite(path, obj):
@@ -561,7 +622,7 @@ def gsPklWrite(path, obj):
     return blob.upload_from_string(pickle.dumps(obj, protocol=4))
 
 
-# In[48]:
+# In[181]:
 
 
 def gsPklRead(path):
@@ -571,7 +632,7 @@ def gsPklRead(path):
 
 # # Pickle
 
-# In[49]:
+# In[182]:
 
 
 def pklwrite(path, obj):
@@ -582,7 +643,7 @@ def pklwrite(path, obj):
             return pickle.dump(obj, f, protocol=4)
 
 
-# In[50]:
+# In[183]:
 
 
 def pklread(path):
@@ -593,7 +654,7 @@ def pklread(path):
             return pickle.load(f)
 
 
-# In[51]:
+# In[184]:
 
 
 if is_testing():
@@ -606,7 +667,7 @@ if is_testing():
 
 # # Images
 
-# In[52]:
+# In[185]:
 
 
 def imread(path):
@@ -640,7 +701,7 @@ def impad(img, top=0, bottom=0, left=0, right=0, color=255):
     return np.pad(img, [(top, bottom), (left, right), (0, 0)], 'constant', constant_values=color)
 
 
-# In[53]:
+# In[186]:
 
 
 def imshow(array, scale=1):
@@ -655,7 +716,7 @@ def imshow(array, scale=1):
         cv2.waitKey()
 
 
-# In[54]:
+# In[187]:
 
 
 if is_testing():
@@ -663,14 +724,14 @@ if is_testing():
     imshow(img)
 
 
-# In[55]:
+# In[188]:
 
 
 if is_testing():
     imshow('http://via.placeholder.com/70.png')
 
 
-# In[56]:
+# In[189]:
 
 
 def imscaleNN(img, s):
@@ -680,7 +741,7 @@ def imscaleBic(img, s):
     return cv2.resize(img, None, fx=s, fy=s, interpolation=cv2.INTER_CUBIC)
 
 
-# In[57]:
+# In[190]:
 
 
 if is_testing():
@@ -699,7 +760,7 @@ if is_testing():
         imshow(out_large_interbic)
 
 
-# In[58]:
+# In[191]:
 
 
 def imRepeat(img, height, width):
@@ -713,7 +774,7 @@ def imRepeat(img, height, width):
     return img
 
 
-# In[59]:
+# In[192]:
 
 
 if is_testing():
@@ -721,33 +782,33 @@ if is_testing():
     imshow(imRepeat(img_random_color_3x5, 7, 11), scale=10)
 
 
-# In[60]:
+# In[193]:
 
 
 def imNewWhite(height, width):
     return np.ones((height, width, 3)).astype(np.uint8) * 255
 
 
-# In[61]:
+# In[194]:
 
 
 if is_testing(): imshow(imNewWhite(10, 500) - 30)
 
 
-# In[62]:
+# In[195]:
 
 
 def imNewBlack(height, width):
     return np.zeros((height, width, 3)).astype(np.uint8)
 
 
-# In[63]:
+# In[196]:
 
 
 if is_testing(): imshow(imNewBlack(10, 500))
 
 
-# In[64]:
+# In[197]:
 
 
 def imAddNoiseGauss(img, std):
@@ -759,7 +820,7 @@ def imAddNoiseGauss(img, std):
 if is_testing(): imshow(imAddNoiseGauss(imread('http://via.placeholder.com/150.png'), 10))
 
 
-# In[65]:
+# In[198]:
 
 
 def imJpgDegradation(img, quality):
@@ -772,7 +833,7 @@ def imJpgDegradation(img, quality):
 if is_testing(): imshow(imJpgDegradation(imread('http://via.placeholder.com/150.png'), 15))
 
 
-# In[66]:
+# In[199]:
 
 
 def imCropCenter(img, size):
@@ -787,35 +848,58 @@ def imCropCenter(img, size):
     return img[h_start:h_end, w_start:w_end]
 
 
-# In[67]:
+# In[200]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 10))
 
 
-# In[68]:
+# In[201]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 49))
 
 
-# In[69]:
+# In[202]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 50))
 
 
-# In[70]:
+# In[203]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 100))
 
 
-# In[71]:
+# In[204]:
+
+
+def imCropRandom(img, size):
+    h, w, c = img.shape
+
+    h_start = np.random.randint(0, h-size)
+    h_end = h_start + size
+
+    w_start = np.random.randint(0, h-size)
+    w_end = w_start + size
+
+    return img[h_start:h_end, w_start:w_end]
+
+
+# In[205]:
+
+
+if is_testing(): 
+    imshow(imCropRandom(imread('http://via.placeholder.com/40.png'), 10))
+    imshow(imCropRandom(imread('http://via.placeholder.com/40.png'), 39))
+
+
+# In[206]:
 
 
 def imGallery(imgs, pad=0):
@@ -846,7 +930,7 @@ def imGallery(imgs, pad=0):
     return out
 
 
-# In[72]:
+# In[207]:
 
 
 if is_testing():
@@ -854,49 +938,49 @@ if is_testing():
     listOfDummyImagesBlackToWhite = [imNewBlack(10, 10) + 25 * i for i in range(10)]
 
 
-# In[73]:
+# In[208]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:1]))
 
 
-# In[74]:
+# In[209]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:2]))
 
 
-# In[75]:
+# In[210]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:2], pad=1))
 
 
-# In[76]:
+# In[211]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:5], pad=1))
 
 
-# In[77]:
+# In[212]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:9], pad=1))
 
 
-# In[78]:
+# In[213]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:10], pad=1))
 
 
-# In[79]:
+# In[214]:
 
 
 if is_testing():
@@ -906,14 +990,14 @@ if is_testing():
 
 # # Files
 
-# In[80]:
+# In[215]:
 
 
 def fiFindByWildcard(wildcard):
     return natsort.natsorted(glob.glob(wildcard, recursive=True))
 
 
-# In[81]:
+# In[216]:
 
 
 if is_testing():
@@ -942,7 +1026,7 @@ if is_testing():
         imshow(imGallery(imgs))
 
 
-# In[82]:
+# In[217]:
 
 
 def fiRemove(path):
@@ -970,7 +1054,7 @@ def fiAgeHours(path):
     return (time.time() - os.stat(path)[stat.ST_MTIME]) / 3600
 
 
-# In[83]:
+# In[218]:
 
 
 if is_testing():
@@ -1002,7 +1086,7 @@ if is_testing():
         print("files: " + "\n".join(fiFindByWildcard(os.path.join(d, "**", "*"))))
 
 
-# In[84]:
+# In[219]:
 
 
 def fiUnzip(zip_path, out_dir=None, remove=False):
@@ -1017,7 +1101,7 @@ def fiUnzip(zip_path, out_dir=None, remove=False):
         zip.extractall(out_dir)
 
 
-# In[85]:
+# In[220]:
 
 
 def fiJoin(*args):
@@ -1029,7 +1113,7 @@ if is_testing():
 
 # # Jupyter
 
-# In[86]:
+# In[221]:
 
 
 def jnNbFullWidth():
@@ -1043,7 +1127,7 @@ def jnPandasSettings():
     pd.set_option('display.max_colwidth', None)
 
 
-# In[87]:
+# In[222]:
 
 
 def jnImageRenderingPixelated():
@@ -1055,7 +1139,7 @@ def jnImageRenderingAuto():
     display(HTML('<style type = text/css> img { image-rendering: auto; } </style>'))
 
 
-# In[88]:
+# In[223]:
 
 
 def jnDefault():
@@ -1073,7 +1157,7 @@ if is_testing():
 
 
 
-# In[89]:
+# In[224]:
 
 
 def jnHtmlRow(strList, width=100):
@@ -1086,7 +1170,7 @@ if is_testing():
     
 
 
-# In[90]:
+# In[225]:
 
 
 def jnImgToHtmlB64(img):
@@ -1121,7 +1205,7 @@ def jnImageRow(imgs, names=None, width=100, external=False, title_img=None, titl
     display(HTML(html))
 
 
-# In[91]:
+# In[226]:
 
 
 if is_testing():
@@ -1129,7 +1213,7 @@ if is_testing():
    
 
 
-# In[92]:
+# In[227]:
 
 
 if is_testing():
@@ -1139,7 +1223,7 @@ if is_testing():
     jnImageRow(imgs, external=True, title='test')
 
 
-# In[93]:
+# In[228]:
 
 
 
@@ -1149,7 +1233,7 @@ if is_testing():
 
 # # Shell
 
-# In[94]:
+# In[229]:
 
 
 def shell(cmd):
@@ -1164,7 +1248,7 @@ def shell(cmd):
 
 # # GIT
 
-# In[95]:
+# In[230]:
 
 
 def gitGetHash():
@@ -1174,7 +1258,7 @@ if is_testing():
     print(gitGetHash())
 
 
-# In[96]:
+# In[231]:
 
 
 def gitGetBranch():
@@ -1184,7 +1268,7 @@ if is_testing():
     print(gitGetBranch())
 
 
-# In[97]:
+# In[232]:
 
 
 def gitGetMessage():
@@ -1194,7 +1278,7 @@ if is_testing():
     print(gitGetMessage())
 
 
-# In[98]:
+# In[233]:
 
 
 def gitGetAllFiles():
@@ -1204,7 +1288,7 @@ if is_testing():
     print(gitGetAllFiles())
 
 
-# In[99]:
+# In[234]:
 
 
 def gitListByTime(path=None):
@@ -1220,7 +1304,7 @@ if is_testing():
     print(gitListByTime()[-3:])
 
 
-# In[100]:
+# In[235]:
 
 
 def shellSortDirsBySize():
@@ -1232,14 +1316,14 @@ if is_testing():
 
 # # Package Files
 
-# In[101]:
+# In[236]:
 
 
 if is_testing():
     package_files = {}
 
 
-# In[102]:
+# In[237]:
 
 
 def shell(cmd):
@@ -1247,7 +1331,7 @@ def shell(cmd):
     return result.stdout.decode()
 
 
-# In[103]:
+# In[238]:
 
 
 if is_testing():
@@ -1279,7 +1363,7 @@ setup(
 )""".format(requirements=", ".join(["'" + r + "'" for r in requirements]), version=version)
 
 
-# In[104]:
+# In[239]:
 
 
 if is_testing():
@@ -1289,7 +1373,7 @@ description-file = README.md
 """
 
 
-# In[105]:
+# In[240]:
 
 
 if is_testing():
@@ -1312,14 +1396,14 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE."""
 
 
-# In[106]:
+# In[241]:
 
 
 if is_testing():
     package_files['README.md'] = ""
 
 
-# In[107]:
+# In[242]:
 
 
 if is_testing():
@@ -1328,7 +1412,7 @@ if is_testing():
     
 
 
-# In[108]:
+# In[243]:
 
 
 if is_testing():
@@ -1339,7 +1423,7 @@ lx/__init__.py
 """
 
 
-# In[109]:
+# In[244]:
 
 
 if is_testing():
